import json
from .base import HomeServerApp


class Transmission(HomeServerApp):
    def setup(self):
        self.container.stop()
        set = json.load((self.path / 'config' / 'settings.json').read_bytes())
        set['rpc-whitelist-enabled'] = False
        (self.path / 'config' / 'settings.json').write_bytes(json.dumps(set))
        self.compose.start()



from bacchus.base import HomeServerApp


class CertManager(HomeServerApp):
    def setup(self):
        cmd = [
            '--dns', 'gandiv5', '-a', '-d', f'private.{self.domain}',
            '--email', f'{self.meta["nextcloud_username"]}', 'run'
        ]
        if (self.path / 'certificates' /
                f'private.{self.domain}.key').exists():
            self.logger.debug('Initial certificates already exists')
        else:
            self.logger.debug('Setting up certificates with lego')
            self.logger.debug(
                self.client.containers.run(
                    'goacme/lego',
                    command=cmd,
                    environment={'GANDIV5_API_KEY': self.meta['dns_api_key']},
                    volumes={
                        self.path.absolute(): {
                            'bind': '/.lego',
                            'mode': 'rw'
                        }
                    },
                    auto_remove=True,
                    detach=False))
            self.logger.debug('Set up certificates with lego')

    def wait_for_status(self):
        # Not actually needed...
        return True



import configparser
import json
from .base import TEMPLATES
from .base import HomeServerApp


class Medusa(HomeServerApp):
    @property
    def config_file(self):
        return self.path / 'config.ini'

    def setup_nginx(self):
        config = configparser.ConfigParser()
        config.read(str(self.config_file))
        config.set('General', 'web_root', '/tv')
        with open(self.config_file, 'w') as fileo:
            config.write(fileo)

    def setup_indexers(self):
        config = configparser.ConfigParser()
        config.read(str(self.config_file))
        api_key = json.loads((self.path / '..' / 'jackett' / 'Jackett' /
                              'ServerConfig.json').read_text())['APIKey']

        for prov in (TEMPLATES / 'jackett' / 'Indexers').glob('*.json'):
            local = dict(name=prov.stem,
                         cat_ids='5010, 5030, 5040',
                         api_key=api_key,
                         search_mode='eponly',
                         search_fallback=0,
                         enable_daily=1,
                         enable_backlog=1,
                         enable_manualsearch=1,
                         enable_search_delay=0,
                         ratio='',
                         search_delay=480,
                         minseed=1,
                         url=(f'https://jackett:9117/trackers/api/v2.0/'
                              'indexers/{prov.stem}/results/torznab/'))
            if not config.has_section(prov.stem.upper()):
                config.add_section(prov.stem.upper())
            config[prov.stem.upper()].update(
                {f'{prov.stem}{prop}': val
                 for prop, val in local.items()})
            if not config.has_section('TORRENT'):
                config.add_section('TORRENT')

            config['TORRENT'].update(
                dict(torrent_username='',
                     torrent_password='',
                     torrent_host=f'https://{self.domain}',
                     torrent_path='',
                     torrent_seed_time=3,
                     torrent_paused=0,
                     torrent_high_bandwidth=0,
                     torrent_label='',
                     torrent_label_anime='',
                     torrent_verify_cert=0,
                     torrent_rpcurl='transmission/rpc',
                     torrent_auth_type='',
                     torrent_seed_location=''))

        with open(self.config_file, 'w') as fileo:
            config.write(fileo)

    def setup(self):
        self.container.stop()
        self.setup_nginx()
        self.setup_indexers()
        self.compose.start()



import netifaces as ni
from contextlib import suppress
from pathlib import Path
from bacchus.base import HomeServerApp

import requests


class OpenVPN(HomeServerApp):
    def setup(self):
        self.setup_dns()
        try:
            self.logger.debug(
                self.run("ovpn_genconfig", "-u",
                         f"udp://public.{self.domain}"))
            with suppress(Exception):
                self.logger.debug(
                    self.run(
                        'bash', '-c',
                        f'echo public.{self.domain}|ovpn_initpki nopass'))

            self.compose.start()
            with suppress(Exception):
                self.logger.debug(
                    self.run('easyrsa', 'build-client-full',
                             self.meta['nextcloud_username'], 'nopass'))
            response = self.run('ovpn_getclient',
                                self.meta['nextcloud_username'])
            Path('vpn_client.config').write_bytes(response)
        except Exception as err:
            self.logger.exception('could not create openvpn config')

    def run(self, *cmd):
        volumes = {
            self.path.absolute(): {
                'bind': '/etc/openvpn',
                'mode': 'rw'
            }
        }
        return self.client.containers.run('kylemanna/openvpn',
                                          command=cmd,
                                          tty=True,
                                          volumes=volumes,
                                          detach=False)

    def setup_dns(self):
        default_iface = next(a for a in ni.interfaces() if any(
            a.startswith(b) for b in ('eth', 'enp')))
        iface = self.meta.get('iface') or default_iface
        private_ip = ni.ifaddresses(iface)[ni.AF_INET][0]['addr']
        public_ip = requests.get('https://api.ipify.org').text
        self.logger.debug(
            f'Setting up public and private for {public_ip}, {private_ip}')
        self.logger.debug(self.create_dns('public', public_ip))
        self.logger.debug('Finished public config')
        self.logger.debug(self.create_dns('private', private_ip))
        self.logger.debug('Finished private config')

    def create_dns(self, domain, addr):
        cmd = [
            "lexicon", "gandi", "create", self.domain, "A", "--auth-token",
            self.meta['dns_api_key'], "--api-protocol", "rest", '--name',
            domain, '--content', addr
        ]
        self.logger.debug(f'Executing {cmd}')

        return self.client.containers.run('analogj/lexicon',
                                          command=cmd,
                                          auto_remove=True,
                                          detach=False)

    def wait_for_status(self):
        return True



from .base import HomeServerApp
from .base import TEMPLATES
from pathlib import Path
import subprocess


class Nginx(HomeServerApp):
    def setup(self):
        self.logger.debug('Setting nginx configuration template')
        (self.path / 'nginx.conf').write_text(
            (TEMPLATES / 'nginx.tpl').read_text().format(domain=self.domain))



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['bacchus']

package_data = \
{'': ['*'], 'bacchus': ['templates/*', 'templates/jackett/Indexers/*']}

install_requires = \
['cleo>=0.7.6,<0.8.0',
 'dns-lexicon>=3.3.17,<4.0.0',
 'docker>=4.1,<5.0',
 'netifaces>=0.10.9,<0.11.0',
 'requests>=2.22.0,<3.0.0']

entry_points = \
{'console_scripts': ['bacchus = bacchus.cli:main']}

setup_kwargs = {
    'name': 'bacchus',
    'version': '0.1.28',
    'description': 'Home Server solution based on docker',
    'long_description': None,
    'author': 'David Francos',
    'author_email': 'opensource@davidfrancos.net',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
